// Original idea from Generative Gestaltung (http://www.generative-gestaltung.de)
// Tweak: HEX0x6C

/**
 * generating a drawing with chars from a text by analysing live video
 * 
 * MOUSE
 * position x          : drawing speed
 * position y          : diffusion
 *
 * KEYS
 * q                   : stop drawing
 * w                   : continue drawing
 * s                   : save png
 * r                   : start record pdf
 * e                   : end record pdf
 * p                   : switch between random and Perlin noise
 * c                   : switch between char and line mode
 * UP                  : increase font size
 * DOWN                : decrease font size
 *
 */

import processing.video.*;
import processing.pdf.*;
import java.util.Calendar;
import processing.sound.*;


boolean savePDF = false;
Capture video;
int x, y, xn, yn;
float diffusion = 50;
PFont font;
String inputText;
int counter = 0;
int maxFontSize = 30;
int minFonSize = 2;
float noi1, noi2; // Perlin noise 
boolean perlin = true; 
boolean character = true;
float speed = 10;
WhiteNoise noise;
SoundFile mp3;
Boolean whiteNoise = false;
Boolean music = false;



void setup() {
  size(640, 480);
  background(255);
  x = width/2;
  y = height/2;
  video = new Capture(this, width, height, 30);
  video.start();
  font = createFont("Times", 10);
  inputText = join(loadStrings("Alice.txt"), "");
  noi1 = random(100);
  noi2 = random(100);
  // Create the noise generator
  noise = new WhiteNoise(this);
  mp3 = new SoundFile(this, "Glass_Glassworks - Opening.mp3");
}


void draw() {
  smooth();
  noFill();

  for (int j=0; j <= mouseX/speed; j++) { // mouseX controls speed of drawing

    // get pixel from video image
    if (video.available()) video.read();
    video.loadPixels();
    int pixelIndex = ((video.width-1-x) + y*video.width);
    color c = video.pixels[pixelIndex];

    // compute next pixel
    diffusion = map(mouseY, 0, height, 5, 100);  // mouseY controls diffusion of randomness
    if (perlin) { // noise
      int rx = (int) ((noise(noi1) - 0.5) * diffusion);
      xn = constrain(x + rx, 0, width-1);
      int ry = (int) ((noise(noi2) - 0.5) * diffusion);
      yn = constrain(y + ry, 0, height-1);
      // increase noise parameters
      noi1 = noi1 + 0.05;
      noi2 = noi2 + 0.05;
    } else {
      // randomness
      int rx = (int) random(-diffusion, diffusion);
      xn = constrain(x + rx, 0, width-1);
      int ry = (int) random(-diffusion, diffusion);
      yn = constrain(y+ry, 0, height-1);
    }

    // draw image
    if (character) { // draw with characters
      fill(c);
      int greyscale = round(red(c)*0.222 + green(c)*0.707 + blue(c)*0.071);
      float fontSize = map(greyscale, 0, 255, maxFontSize, minFonSize);
      textFont(font, fontSize);
      char letter = inputText.charAt(counter);
      text(letter, x, y);
      counter++;
      if (counter > inputText.length()-1) counter = 0;
    } else { // draw with lines
      stroke(c);
      line(x, y, xn, yn);
    }

    // update current position 
    x = xn;
    y = yn;
  }
}


void keyPressed() {
  switch(key) {
  case BACKSPACE:
    background(255);
    noi1 = random(100);
    noi2 = random(100);
    break;
  }
}


void keyReleased() {
  if (key == 's' || key == 'S') saveFrame(timestamp()+"_##.png");

  if (key == 'r' || key == 'R') {  
    background(255);
    beginRecord(PDF, timestamp()+".pdf");
  }
  if (key == 'e' || key == 'E') {  
    endRecord();
  }

  if (key == 'p' || key == 'P') {  
    perlin = !perlin;
  }

  if (key == 'c' || key == 'C') {  
    character = !character;
  }

  if (key == 'n' || key == 'n') {
    whiteNoise = !whiteNoise;
    if (whiteNoise) noise.play(); 
    else noise.stop();
  }
  if (key == 'm' || key == 'M') {
    music = !music;
    if (music) mp3.play(); 
    else mp3.pause();
  }

  if (key == 'q' || key == 'S') noLoop();
  if (key == 'w' || key == 'W') loop();

  if (keyCode == UP) maxFontSize = min(maxFontSize+2, 60);
  if (keyCode == DOWN) maxFontSize = max(maxFontSize-2, minFonSize);
}

// timestamp
String timestamp() {
  Calendar now = Calendar.getInstance();
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", now);
}
